# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/Run_raw_import.py $
# Last committed:     $Revision: 347639 $; $Author: svcdms $; $Date: 2019-10-02 16:39:05 +0200 (wo, 02 okt 2019) $
#
from dmsLogger.DmsLogger import DmsLogger
from fewsServices.UploadFewsPy import UploadFewsPy
from misc.Utils import Utils
from misc.REMatcher import REMatcher
from os import walk, path, makedirs, remove, rename
from re import sub
from readRaws import ReadRaws
import getopt
import sys


def move_file_to_failed_folder(source, settings):
    target = sub(settings["import_folder"], settings["import_failed"], source)
    try:
        makedirs(path.dirname(target))
    except FileExistsError:
        pass
    try:
        rename(source, target)
    except FileExistsError:
        remove(target)
        rename(source, target)


def run_import(settings):
    reader = getattr(ReadRaws, settings["reader_object"])
    logger = DmsLogger.set_logger(settings["log_folder"], settings["logger"], settings["logfile"])
    logger.info("Run_raw_import.run_import: $Revision: 347639 $")

    for dirName, subdirList, fileList in walk(settings["import_folder"]):
        logger.info("{}: Process files in folder {}".format(settings["process_name"], dirName))
        for fname in fileList:
            full_path = path.join(dirName, fname)

            # do not try to import the STB_AVG files
            if "STB_AVG_" in fname:
                logger.debug("Run_raw_import.run_import: removing AVG file {}".format(full_path))
                remove(full_path)
                continue

            # do not try to import the ADV_*.dat files
            m = REMatcher(fname)
            if m.search('ADV_\w*\.dat'):
                logger.debug("Run_raw_import.run_import: removing ADV.dat file {}".format(full_path))
                remove(full_path)
                continue

            size = path.getsize(full_path)
            if size == 0:
                logger.warning("{}: file is empty {}".format(settings["process_name"], fname))
                logger.warning("{}: moving file to {}".format(settings["process_name"], settings["import_failed"]))
                move_file_to_failed_folder(full_path, settings)
                continue

            location = Utils.get_location_from_filename(fname, settings["location_pattern"])
            if location == "":
                logger.warning("{}: location could not be retrieved from file name {}".format(settings["process_name"], fname))
                logger.warning("{}: moving file to {}".format(settings["process_name"], settings["import_failed"]))
                move_file_to_failed_folder(full_path, settings)
                continue

            logger.info("{}: Processing file {}".format(settings["process_name"], full_path))
            data_reader = reader(full_path, logger, settings)

            if data_reader.get_data_read_successful():
                location = Utils.get_location_from_filename(fname, settings["location_pattern"])
                if location != "":
                    logger.debug("{}: location from file name {}".format(settings["process_name"], location))
                    upload = UploadFewsPy(logger, settings["pi_service"], settings["filter"])
                    if upload.write_series(logger, data_reader, settings["moduleInstance"], location):
                        remove(full_path)
                    else:
                        move_file_to_failed_folder(full_path, settings)
                else:
                    logger.warning("{}: location could not be retrieved from file name {}".format(settings["process_name"], fname))
            else:
                # move file to the import failed folder
                logger.warning("{}: moving file to {}".format(settings["process_name"], settings["import_failed"]))
                move_file_to_failed_folder(full_path, settings)


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("Run_raw_import.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("Run_raw_import.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_process_settings(settings_file)
        run_import(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
